const S = 'start';
const W = 'wall';
const F = 'floor';

function flip (t) {
	const flipped = [];
	console.log(t.length);
	for (let idx = 0; idx < t[0].length; idx++) {
		flipped[idx] = [];
		for (let jdx = 0;jdx < t.length; jdx++) {
			flipped[idx][jdx] = t[jdx][idx];
		}
	}
	return flipped;
}

const tiles = flip([
	[W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W],
	[W, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, W],
	[W, F, W, W, W, W, W, F, W, W, W, W, F, W, W, W, W, W, F, W],
	[W, F, F, F, F, F, F, F, W, F, F, W, F, F, F, F, F, F, F, W],
	[W, F, W, W, W, W, W, W, W, F, F, W, W, W, W, W, W, W, F, W],
	[W, F, F, F, F, F, F, F, W, F, F, W, F, F, F, F, F, F, F, W],
	[W, F, W, W, W, W, W, F, W, F, F, W, F, W, W, W, W, W, F, W],
	[W, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, W],
	[W, F, W, W, W, W, W, W, W, F, F, W, W, W, W, W, W, W, F, W],
	[W, F, F, F, F, F, F, F, F, F, S, F, F, F, F, F, F, F, F, W],
	[W, F, W, W, W, F, W, W, W, W, W, W, W, W, F, W, W, W, F, W],
	[W, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, W],
	[W, F, W, W, W, W, W, W, W, F, F, W, W, W, W, W, W, W, F, W],
	[W, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, F, W],
	[W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W, W]
]);

const level = [];

for (let idx = 0; idx < tiles.length; idx++) {
	let row = tiles[idx];
	let levelRow = [];
	for (let jdx = 0; jdx < row.length; jdx++) {
		const tile = row[jdx];
		const block = {
			type: tile
		};
		if (tile !== W) {
			const left = tiles[idx - 1][jdx];
			const right = tiles[idx + 1][jdx];
			const up = tiles[idx][jdx - 1];
			const down = tiles[idx][jdx + 1];
			block.directions = {};
			block.directions.left = left !== W;
			block.directions.right = right !== W;
			block.directions.up = up !== W;
			block.directions.down = down !== W;
			block.position = {x: idx, y: jdx};
		}
		levelRow.push(block);
	}
	level.push(levelRow);
}

// const l2 = [];
// for (let jdx = 0; jdx < level[0].length; jdx++) {
// 	l2[jdx]=[];
// 	for (let idx = 0; idx < level.length; idx++) {
// 		l2[jdx][idx]=level[idx][jdx];
// 	}
// }
// level = l2;

// states
const KEY_NONE = 0;
const KEY_PRESS = 1;
const KEY_HELD = 2;
const KEY_RELEASE = 3;
// key
const KEY_UP = 38;
const KEY_DOWN = 40;
const KEY_LEFT = 37;
const KEY_RIGHT = 39;
const KEY_RUN = 16;
const KEY_FIRE = 90;

const heldButtons = {};
const buttonPresses = {};

function keyDown(key) {
	if (!heldButtons[key.which]) {
		buttonPresses[key.which] = KEY_PRESS;
		heldButtons[key.which] = true;
	}
}

function keyUp(key) {
	// console.log('what:', key.which, key.key);
	if (heldButtons[key.which]) {
		buttonPresses[key.which] = KEY_RELEASE;
		heldButtons[key.which] = false;
	}
}

function updateInput() {
	for (const button in buttonPresses) {
		if (heldButtons[button]) {
			if (buttonPresses[button] === KEY_PRESS) {
				buttonPresses[button] = KEY_HELD;
			}
		}
		if (buttonPresses[button] === KEY_RELEASE) {
			buttonPresses[button] = KEY_NONE;
		}
	}
}

const DIR_UP = 'up';
const DIR_DOWN = 'down';
const DIR_LEFT = 'left';
const DIR_RIGHT = 'right';

const enemy = {
	x: 18,
	y: 5,
	walking: false,
	direction: null,
	rate: 128,
	tile: level[18][5],
	stop: function(tile) {
		this.tile = tile;
		this.walking = false;
		this.direction = null;
		this.x = tile.position.x;
		this.y = tile.position.y;
	},
	update: function(delta) {
		if (this.walking) {
			switch(this.direction) {
				case DIR_UP:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.y - 1 < this.y) {
							this.y -= delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x][this.tile.position.y - 1]);
						}
					}
					break;
				case DIR_DOWN:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.y + 1 > this.y) {
							this.y += delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x][this.tile.position.y + 1]);
						}
					}
					break;
				case DIR_LEFT:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.x - 1 < this.x) {
							this.x -= delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x - 1][this.tile.position.y]);
						}
					}
					break;
				case DIR_RIGHT:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.x + 1 > this.x) {
							this.x += delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x + 1][this.tile.position.y]);
						}
					}
					break;
			}
		} else {
		}
	},
	draw: function(_2d,delta) {
		_2d.drawImage(foe,
			0, 0, 13, 15,
			(this.x * 16) + 2, (this.y * 16), 13, 15
		);
	}
};

const player = {
	x: 1,
	y: 6,
	walking: false,
	direction: null,
	rate: 768,
	tile: level[1][6],
	stop: function(tile) {
		this.tile = tile;
		this.walking = false;
		this.direction = null;
		this.x = tile.position.x;
		this.y = tile.position.y;
	},
	update: function(delta) {
		if (this.walking) {
			switch(this.direction) {
				case DIR_UP:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.y - 1 < this.y) {
							this.y -= delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x][this.tile.position.y - 1]);
						}
					}
					break;
				case DIR_DOWN:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.y + 1 > this.y) {
							this.y += delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x][this.tile.position.y + 1]);
						}
					}
					break;
				case DIR_LEFT:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.x - 1 < this.x) {
							this.x -= delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x - 1][this.tile.position.y]);
						}
					}
					break;
				case DIR_RIGHT:
					if (this.tile.directions[this.direction]) {
						if (this.tile.position.x + 1 > this.x) {
							this.x += delta / this.rate;
						} else {
							this.stop(level[this.tile.position.x + 1][this.tile.position.y]);
						}
					}
					break;
			}
		} else {
		}
	},
	draw: function(_2d,delta) {
		// _2d.fillRect(this.x * 16, this.y * 16, 16, 16);
		_2d.drawImage(char,
			0, 0, 11, 12,
			(this.x * 16) + 2, (this.y * 16) + 2, 11, 12
		);
	},
	walk: function(direction) {
		if (this.walking) {
			switch (this.direction) {
				case DIR_UP:
					if (direction === DIR_DOWN) {
						this.direction = DIR_DOWN;
						this.tile = level[this.tile.position.x][this.tile.position.y - 1];
					}
				break;
				case DIR_DOWN:
					if (direction === DIR_UP) {
						this.direction = DIR_UP;
						this.tile = level[this.tile.position.x][this.tile.position.y + 1];
					}
				break;
				case DIR_LEFT:
					if (direction === DIR_RIGHT) {
						this.direction = DIR_RIGHT;
						this.tile = level[this.tile.position.x - 1][this.tile.position.y];
					}
				break;
				case DIR_RIGHT:
					if (direction === DIR_LEFT) {
						this.direction = DIR_LEFT;
						this.tile = level[this.tile.position.x + 1][this.tile.position.y];
					}
				break;
			}
		} else {
			if (this.tile.type !== W) {
				if (this.tile.directions[direction]) {
					this.walking = true;
					this.direction = direction;
				}
			}
		}
	}
};

function logic(delta) {
	if (heldButtons[KEY_UP]) {
		player.walk(DIR_UP);
	}
	if (heldButtons[KEY_DOWN]) {
		player.walk(DIR_DOWN);
	}
	if (heldButtons[KEY_LEFT]) {
		player.walk(DIR_LEFT);
	}
	if (heldButtons[KEY_RIGHT]) {
		player.walk(DIR_RIGHT);
	}
	player.update(delta);
	enemy.update(delta);
}

function scene(_2d, delta) {
	_2d.clearRect(0,0,640,480);
	level.forEach((row, idx) => {
		row.forEach((col, jdx) => {
			if(col.type === W) {
				// walls are green
				_2d.fillStyle = '#666';
				// _2d.globalAlpha = 1;
			} else {
				_2d.fillStyle = '#333';
				// _2d.globalAlpha = 1 / 8;
			}
			_2d.fillRect(16 * idx, 16 * jdx, 16, 16);
		});
	});
	_2d.fillStyle = '#ff0';
	player.draw(_2d,delta);
	enemy.draw(_2d,delta);
	// _2d.fillRect(player.x * 16, player.y * 16, 16, 16);
	// _2d.fillRect(player.x - 8, player.y - 8, 16, 16);
	_2d.fillStyle = 'white';
	_2d.fillText('player: \'' + player.x + ':' + player.y + '\'', 10, 220);
	// _2d.fillText('delta: ' + delta + '\'', 10, 220);
	// _2d.drawImage(astr, player.x - 20, player.y - 30);
}

addEventListener('load', (event) => {
	const canvas = window.document.getElementById('canvas');
	_2d = canvas.getContext('2d');
	_2d.imageSmoothingEnabled = false
	addEventListener('keyup', keyUp);
	addEventListener('keydown', keyDown);
	let delta = 0;
	let previous = 0;
	function frame(frametime) {
		delta = frametime - previous;
		logic(delta);
		scene(_2d, delta);
		updateInput();
		previous = frametime;
		requestAnimationFrame(frame);
	}
	requestAnimationFrame(frame);
});
